#!/usr/bin/env python
# vim: set ts=4 sw=4 sts=0 :
# GameScore.py :
#
# Copyright (C) 2007, Shu KONNO <owa@bg.wakwak.com>
# This program distributed under GPL (GNU General Public License).
import httplib
from ScoreXML import ScoreXML
from FormatCheck import FormatCheck
import os
import gettext 

class GameScore(ScoreXML):
	def __init__(self, path, client):
		ScoreXML.__init__(self, os.path.join(path, 'score.xml'))
		self.__http_header = {}
		sysname, host, rel, ver, arch = os.uname()
		self.__http_header['User-Agent'] = '%s (%s %s %s)' % (client, sysname, rel, arch)

	def wgetScore(self, uri):
		#print 'wgetScore:', uri
		s = httplib.urlsplit(uri)
		if s[0] == 'http':
			host = s[1]
			adrs = s[2]
			conn = httplib.HTTPConnection(host)
			conn.request("GET", adrs, body=None, headers=self.__http_header)
			r = conn.getresponse()
			if r.status != 200: 
				print r.status, r.reason
				conn.close()
			else:
				s = r.read()
				conn.close()
				info = FormatCheck(s)
				if info:
					self.__setInfo(info, uri=uri)
				else:
					self.construct(s)

	def saveScore(self, path=''):
		if path:
			self.setFilePath(path)
		self.savefile(path)

	def loadScore(self, path=None):
		if not path:
			self.loadfile()
		else:
			f = open(path, 'rb')
			s = f.read()
			f.close()
			info = FormatCheck(s)
			if info:
				self.__setInfo(info, path=path)
			else:
				self.construct(s)

	def __getLocalFile(self, path):
		#print 'getLocalFile:', path
		f = open(path, 'rb')
		s = f.read()
		f.close()
		info = FormatCheck(s)
		if info:
			self.__setInfo(info, path=path)
		else:
			self.loadfile(path)

	def __setInfo(self, info, path=None, uri=None):
		if uri:
			self.setURI(uri)
			self.setFilePath('')
		if path:
			self.setURI('')
			self.setFilePath(path)
		self.setTitle(info.getTitle())
		self.setComment(info.getComment())
		self.setAuthor('')
		self.setWhite(info.getWhite())
		self.setBlack(info.getBlack())
		self.setGameType('H0')
		self.setMoving(info.getMoving())


if __name__ == '__main__':
	f = GameScore('./', 'TestClient')
	f.loadScore('kifu/kifu-6.12.txt')
	f.dump()
	f.savefile()
	f.loadfile()
	print 'score=', f.getScore()

