#!/usr/bin/env python
# vim: set ts=4 sw=4 sts=0 fileencoding=utf-8 enc=utf-8 :
# KifuFormat.py:
#
# Copyright (C) 2007, Shu KONNO <owa@bg.wakwak.com>
# This program distributed under GPL (GNU General Public License).
#
# Kifu for Windows V6.22
# http://homepage2.nifty.com/kakinoki_y/
from ScoreInterface import ScoreInterface
import re

_KIFU_V6 = u'# ----  Kifu for Windows V6.* 棋譜ファイル  ----'

class KifuFormat(ScoreInterface):
	def __init__(self):
		ScoreInterface.__init__(self)

	def check(self, s):
		encodes = ('utf-8', 'shift_jis', 'euc-jp')
		for encode in encodes:
			u = unicode(s, encode, 'replace')
			if self.__check(u):
				return True

	def __check(self, s):
		s = s.split('\n')
		k = s[0]
		if re.match(_KIFU_V6, k):
			#print 'Kifu for Windows: yes'
			n = self.__getHeader(s[1:])
			self.__getData(s[n:])
			return True

	def __getHeader(self, rows):
		cnt = 0
		for u in rows:
			cnt += 1
			if u[:2] == u'手数':
				break
			items = u.split(u'：')
			if len(items) >= 2:
				k = items[0]
				v = items[1]
				if k == u'開始日時': pass
				if k == u'終了日時': pass
				if k == u'棋戦': self.setTitle(v)
				if k == u'持ち時間': pass
				if k == u'消費時間': pass
				if k == u'場所': self.setComment(v)
				if k == u'手合割': pass
				if k == u'先手': self.setBlack(v)
				if k == u'後手': self.setWhite(v)
		return cnt

	def __getData(self, rows):
		s = ''
		kc = KifuConverter()
		for u in rows:
			if re.match(r'^ *[0-9]* ', u):
				items = re.split(r' *', u)
				orig = items[2]
				move = kc.convert(orig)
				if move is None:
					break;
				#print move, orig
				s += move
		self.setMoving(s.encode('ascii'))


class KifuConverter(object):
	def __init__(self):
		self.__bw = 1
		self.__pos = ''

	def convert(self, move):
		dict = {
			u'１':'1', u'一':'1',
			u'２':'2', u'二':'2',
			u'３':'3', u'三':'3',
			u'４':'4', u'四':'4',
			u'５':'5', u'五':'5',
			u'６':'6', u'六':'6',
			u'７':'7', u'七':'7',
			u'８':'8', u'八':'8',
			u'９':'9', u'九':'9',
			u'王':'K', u'玉':'K',
			u'飛':'R', u'竜':'R', u'龍':'R',
			u'角':'B', u'馬':'B',
			u'金':'G',
			u'銀':'S',
			u'桂':'N',
			u'香':'L',
			u'歩':'P', u'と':'P',
			u'　':'', u'上':'', u'下':'', u'右':'', u'左':'',
			u'同':'X',
			u'成':'+',
			u'打':'\'',
			u'投':'E',
			u'了':'E',
			u')':'',
		}
		cplus = {'1':'a', '2':'b', '3':'c', '4':'d',
			'5':'e', '6':'f', '7':'g', '8':'h', '9':'i'}
		u = u''
		for c in move:
			u += dict.get(c, c)
		if u != 'EE':
			if u[0] == u'X':
				dst = self.__pos
				u = u[1:]
			else:
				dst = u[:2]
				u = u[2:]
			self.__pos = dst
	
			plus = False
			if u[1] == u'\'':
				src = str(self.__bw) + u[0]
			elif u[1] == u'+':
				src = u[3:]
				plus = True
			elif u[0] == u'+':
				src = u[3:]
			else:
				src = u[2:]
			if plus:
				d = cplus.get(dst[0], '?')
				dst = d + dst[1]
	
			self.__bw ^= 0x03
			return dst + src

if __name__ == '__main__':
	files = ('kifu/kifu-6.12.txt', 'kifu/kifu-6.22.txt', 'kifu/kifu-6.22.utf8')
	for path in files:
		print '\n--- %s ---' % path
		f = open(path, 'rb')
		s = f.read()
		f.close()
		conv = KifuFormat()
		if conv.check(s):
			d = conv.getInfo()
			for item in d.items():
				print '%s: %s' % item

