# vim: set ts=4 sw=4 sts=0 :
#
# ZShogiSite.py
#
# Copyright (C) 2004, Shu KONNO <owa@bg.wakwak.com> All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsuo Ishimoto not be used in advertising or publicity pertaining
# to distribution of the software without specific, written prior permission.
#
# SHU KONNO DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.
#
##############################################################################
"""ZShogiSite: zope japanese shogi site product."""

__version__='$Revision: 0.5.0.0 $'[11:-2]
import os, re
from App.Common import package_home
from OFS.Folder import Folder
from Globals import DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo
from ZShogi import ZShogi
#from ZPlayShogi import ZPlayShogi
from ZEvent import proc_init_event, proc_move_event
import ZView

dtml_addSite = DTMLFile('dtml/ManageAddSite', globals())

def manage_addSite(self, id, title='', REQUEST=None):
	"""Method to add a new ZShogi Site object."""
	obj = ZShogiSite(id, title)
	obj.title = title
	self._setObject(id, obj)
	if REQUEST is not None:
		return self.manage_main(self, REQUEST)

class ZShogiSite(Folder):
	"""ZShogi Site Class"""
	meta_type = "ZShogi Site"

	security = ClassSecurityInfo()
	security.setPermissionDefault('View',('Anonymous','Manager'))
	#security.setDefaultAccess("deny")

	dtml_set_diagram = DTMLFile('dtml/SuiteSetDiagram', globals())
	dtml_move_pieces = DTMLFile('dtml/SuiteMovePieces', globals())

	security.declarePrivate('__init__')
	def __init__(self, id, title=''):
		"""Method to initialize ZShogiSite instance."""
		Folder.__init__(self, id)
		dir = os.path.join(package_home(globals()), 'skel')
		self.__addDTML(dir, 'create_score.dtml', 'create_score', 'create score')
		self.__addDTML(dir, 'create_puzzle.dtml', 'create_puzzle', 'create puzzle')
		self.__addDTML(dir, 'demo.dtml', 'demo', '')
		self.__addDTML(dir, 'edit.dtml', 'edit', '')
		self.__addDTML(dir, 'index_html.dtml', 'index_html', '')
		self.__addDTML(dir, 'index_image.dtml', 'index_image', 'image index')

	security.declarePrivate('__init__')
	def __addDTML(self, dir, fname, id, title):
		path = os.path.join(dir, fname)
		f = open(path)
		body = f.read()
		f.close()
		self.manage_addDTMLMethod(id, title, body)

	security.declareProtected('View', 'show0')
	def show0(self, REQUEST, size=0):
		"""Method to show board image."""
		if REQUEST is not None:
			REQUEST.RESPONSE.setHeader('Content-Type','image/png')
		return ZView.showBoard(REQUEST, size)

	security.declareProtected('View', 'show1')
	def show1(self, REQUEST, size=0):
		"""Method to show blacket-1 image."""
		if REQUEST is not None:
			REQUEST.RESPONSE.setHeader('Content-Type','image/png')
		return ZView.showBlacket1(REQUEST, size)

	security.declareProtected('View', 'show2')
	def show2(self, REQUEST, size=0):
		"""Method to show blacket-2 image."""
		if REQUEST is not None:
			REQUEST.RESPONSE.setHeader('Content-Type','image/png')
		return ZView.showBlacket2(REQUEST, size)

	security.declareProtected('Add ZShogi', 'suiteSetDiagram')
	def suiteSetDiagram(self, REQUEST=None, stype='S0', size=0):
		"""Method to make puzzle."""
		if REQUEST is not None:
			if not REQUEST.has_key('sg_type'):
				self.__initRequest(REQUEST, stype)
			elif REQUEST.has_key('se_save'):
				self.__createShogiObject(REQUEST, stype)
				REQUEST['sg_pick'] = ''
			else:
				proc_init_event(REQUEST, size=size)

			REQUEST.SESSION.set('ss_diag', REQUEST['sg_diag'])
			REQUEST.SESSION.set('ss_pick', REQUEST['sg_pick'])
			return self.dtml_set_diagram(self, REQUEST)

	security.declareProtected('Add ZShogi', 'suiteMovePieces')
	def suiteMovePieces(self, REQUEST=None, stype='H0', size=0):
		"""Method to move pieces."""
		if REQUEST is not None:
			if not REQUEST.has_key('sg_type'):
				self.__initRequest(REQUEST, stype)
			elif REQUEST.has_key('se_save'):
				self.__createShogiObject(REQUEST, stype)
				REQUEST['sg_pick'] = ''
			else:
				proc_move_event(REQUEST, size=size)

			REQUEST.SESSION.set('ss_diag', REQUEST['sg_diag'])
			REQUEST.SESSION.set('ss_pick', REQUEST['sg_pick'])
			return self.dtml_move_pieces(self, REQUEST)

	security.declareProtected('Add ZShogi', 'suitePlayGame')
	def suitePlayGame(self, REQUEST=None, stype='H0', size=0):
		"""Method to play game. @@@@"""
		if REQUEST is not None:
			if not REQUEST.has_key('sg_type'):
				self.__initRequest(REQUEST, stype)
			else:
				proc_move_event(REQUEST, size=size)
				touch = REQUEST.get('sg_pick', '')
				if touch:
					if isinstance(touch, tuple) and touch[0] == '2':
						REQUEST['sg_pick'] = ''
					REQUEST['sg_diag'] = stype
					REQUEST['sg_hist'] = ''

			REQUEST.SESSION.set('ss_diag', REQUEST['sg_diag'])
			REQUEST.SESSION.set('ss_pick', REQUEST['sg_pick'])
			return self.dtml_move_pieces(self, REQUEST)

	security.declareProtected('Add ZShogi', 'suiteSaveButton')
	def suiteSaveButton(self, REQUEST=None):
		"""Method to save."""
		if REQUEST is not None:
			return '<input type="submit" name="se_save" value="Save">'

	security.declarePrivate('__init__')
	def __initRequest(self, REQUEST, stype):
		REQUEST['sg_type'] = stype
		REQUEST['sg_diag'] = stype
		REQUEST['sg_side'] = 1
		REQUEST['sg_hist'] = ''
		REQUEST['sg_numb'] = 0
		REQUEST['sg_pick'] = ''

	security.declarePrivate('__init__')
	def __createShogiObject(self, REQUEST, stype):
		container, id = self.__getCreateID(REQUEST)
		title = REQUEST.get('sd_title', '')
		obj = ZShogi(id, title)
		obj.setTypeDiag(stype, REQUEST['sg_diag'])
		obj.setScore(REQUEST.get('sg_hist', ''))
		list = container.suitePropertyList(REQUEST)
		for c in list:
			obj.manage_addProperty(c[0], c[1], 'string')
		container._setObject(id, obj)

#	security.declareProtected('Add ZShogi', 'createPlayObject')
#	def createPlayObject(self, REQUEST):
#		""" method to create ZPlayShogi object and return it's URL """
#		container, id = self.__getCreateID(REQUEST)
#		obj = ZPlayShogi(id, '')
#		stype = REQUEST['sg_type']
#		sdiag = REQUEST['sg_diag']
#		obj.setTypeDiag(stype, sdiag)
#		obj.setMasterIdent(REQUEST)
#		container._setObject(id, obj)
#		url = "/%s/%s" % (container.absolute_url(1), id)
#		print '@@@@createPlayObject: url=', url
#		return url

	def __getCreateID(self, REQUEST):
		container = self
		if REQUEST.has_key('se_store'):
			store = REQUEST.get('se_store', '')
			#print 'store=', store
			store = self.restrictedTraverse(store)
			if store:
				#print 'restrictedTraverse=', store.getId()
				container = store
		stype = REQUEST['sg_type']
		id = stype + '_001'
		#objs = container.objectValues(['ZShogi', 'ZShogi Play'])
		objs = container.objectValues('ZShogi')
		if objs:
			ob = objs[-1]
			id = ob.id
			m = re.search('\d*$', id)
			if m:
				s = m.group()
				fmt = '%%0%dd' % len(s)
				s = fmt % (int(s) + 1)
				id = id[:m.start()] + s
		return container, id

	security.declareProtected('View', 'suitePropertyData')
	def suitePropertyData(self, REQUEST=None):
		"""Method to get property data."""
		list = []
		if REQUEST is not None:
			items = self.propertyItems();
			for c in items:
				s = c[0]
				if s[:3] == 'sd_':
					d = REQUEST.get(s, '')
					list.append((s, (c[1], d)))
		return list

	security.declareProtected('View', 'suitePropertyList')
	def suitePropertyList(self, REQUEST=None):
		"""Method to get property dict."""
		list = []
		if REQUEST is not None:
			ids = self.propertyIds();
			for id in ids:
				if id[:3] == 'sd_':
					val = REQUEST.get(id, '')
					list.append((id, val))
		return list

	security.declareProtected('View', 'show')
	def show(self, REQUEST):
		"""Method to show current diagram."""
		#print 'show: diag=', REQUEST.diag
		if REQUEST.has_key('diag'):
			REQUEST.RESPONSE.setHeader('Content-Type','image/png')
			return ZView.showDirect(REQUEST.diag)

InitializeClass(ZShogiSite)

