# vim: set ts=4 sw=4 sts=0 :
# Copyright (C) 2004, Shu KONNO <owa@bg.wakwak.com> All Rights Reserved.
from CBoardSpace import CBoardSpace
import PIL.Image
import os
path = os.path.join(os.path.dirname(__file__), 'img')

c_blacks = {
	'K': PIL.Image.open(os.path.join(path, '31x27c/B10.png'), 'r'),
	'X': PIL.Image.open(os.path.join(path, '31x27c/B20.png'), 'r'),
	'R': PIL.Image.open(os.path.join(path, '31x27c/B30.png'), 'r'),
	'B': PIL.Image.open(os.path.join(path, '31x27c/B40.png'), 'r'),
	'G': PIL.Image.open(os.path.join(path, '31x27c/B50.png'), 'r'),
	'S': PIL.Image.open(os.path.join(path, '31x27c/B60.png'), 'r'),
	'N': PIL.Image.open(os.path.join(path, '31x27c/B70.png'), 'r'),
	'L': PIL.Image.open(os.path.join(path, '31x27c/B80.png'), 'r'),
	'P': PIL.Image.open(os.path.join(path, '31x27c/B90.png'), 'r'),
	'+K': PIL.Image.open(os.path.join(path, '31x27c/B11.png'), 'r'),
	'+R': PIL.Image.open(os.path.join(path, '31x27c/B31.png'), 'r'),
	'+B': PIL.Image.open(os.path.join(path, '31x27c/B41.png'), 'r'),
	'+G': PIL.Image.open(os.path.join(path, '31x27c/B51.png'), 'r'),
	'+S': PIL.Image.open(os.path.join(path, '31x27c/B61.png'), 'r'),
	'+N': PIL.Image.open(os.path.join(path, '31x27c/B71.png'), 'r'),
	'+L': PIL.Image.open(os.path.join(path, '31x27c/B81.png'), 'r'),
	'+P': PIL.Image.open(os.path.join(path, '31x27c/B91.png'), 'r'),
}
c_whites = {
	'K': PIL.Image.open(os.path.join(path, '31x27c/W20.png'), 'r'),
	'R': PIL.Image.open(os.path.join(path, '31x27c/W30.png'), 'r'),
	'B': PIL.Image.open(os.path.join(path, '31x27c/W40.png'), 'r'),
	'G': PIL.Image.open(os.path.join(path, '31x27c/W50.png'), 'r'),
	'S': PIL.Image.open(os.path.join(path, '31x27c/W60.png'), 'r'),
	'N': PIL.Image.open(os.path.join(path, '31x27c/W70.png'), 'r'),
	'L': PIL.Image.open(os.path.join(path, '31x27c/W80.png'), 'r'),
	'P': PIL.Image.open(os.path.join(path, '31x27c/W90.png'), 'r'),
	'+K': PIL.Image.open(os.path.join(path, '31x27c/W21.png'), 'r'),
	'+R': PIL.Image.open(os.path.join(path, '31x27c/W31.png'), 'r'),
	'+B': PIL.Image.open(os.path.join(path, '31x27c/W41.png'), 'r'),
	'+G': PIL.Image.open(os.path.join(path, '31x27c/W51.png'), 'r'),
	'+S': PIL.Image.open(os.path.join(path, '31x27c/W61.png'), 'r'),
	'+N': PIL.Image.open(os.path.join(path, '31x27c/W71.png'), 'r'),
	'+L': PIL.Image.open(os.path.join(path, '31x27c/W81.png'), 'r'),
	'+P': PIL.Image.open(os.path.join(path, '31x27c/W91.png'), 'r'),
}

c_hilight = lambda i: i - 100

class Layout:
	def __init__(self, side, W=96, H=128, w=27, h=31):
		self.__side = side
		self.__width = W
		self.__height = H
		self.__xs = w
		self.__ys = h
		self.__collect = []
		self.__postable = []

	def getImage(self, image, parts):
		for list in self.__postable:
			piece = list[0]
			img = parts[piece]
			tmp = list[2:]
			tmp.reverse()
			for pos in tmp:
				image.paste(img, pos, img)
		return image

	def pickImage(self, image, parts, pick):
		for list in self.__postable:
			piece = list[0]
			img = parts[piece]
			tmp = list[2:]
			if pick == piece:
				pos = tmp[0]
				x = pos[0]
				y = pos[1]
				box = image.crop((x, y, x + 27, y + 31))
				box = box.point(c_hilight)
				image.paste(box, pos, img)

	def setPieces(self, pieces):
		self.__setCollect(pieces)
		self.__setPosition()
		self.__postable = []
		for row in self.__collect:
			for s in row[2:]:
				self.__postable.append(s)

	def getPiece(self, pos):
		self.__postable.reverse()
		x, y = pos
		for list in self.__postable:
			piece = list[0]
			for pos in list[2:]:
				x1 = pos[0]
				x2 = x1 + self.__xs
				y1 = pos[1]
				y2 = y1 + self.__ys
				if x > x1 and x < x2 and y > y1 and y < y2:
					return piece
		return None

	def __setCollect(self, pieces):
		self.__collect = []
		for group in ('KRB','GS','NL','P'):
			sub = []
			sum = 0;
			for s in group:
				n = pieces[s]
				if n > 0:
					sum += n
					sub.append([s, n])
			if sum > 0:
				self.__collect.append([group, sum] + sub)

	def __setPosition(self):
		y = (self.__height - self.__ys * len(self.__collect)) / 2
		for row in self.__collect:
			sum = row[1]
			x = 0
			dx = (self.__width - 2) / sum
			if sum < 4:
				x = (self.__width - self.__xs * sum) / 2
				dx = self.__xs;
			self.__setRow(row[2:], x, y, dx)
			y += self.__ys

	def __setRow(self, row, x, y, dx):
		for s in row:
			n = s[1]
			while n > 0:
				if self.__side == 1:
					s.append((x, y))
				elif self.__side == 2:
					ax = self.__width - self.__xs - x
					ay = self.__height - self.__ys - y
					s.append((ax, ay))
				n -= 1
				x += dx

class CxBoard(CBoardSpace):
	def __init__(self):
		CBoardSpace.__init__(self, xo=6, yo=6, xs=34, ys=38, b=c_blacks, w=c_whites)
		self.imgpath = os.path.join(path, 'board38x34/board.png')

	def pasteOnBoard(self, base, img, x=0, y=0):
		""" method to called by board (supper) class """
		base.paste(img, (x + 3, y + 5), img)

	def pickupTarget(self, space, pos, id):
		pick = None
		if id == 0:
			pos = CBoardSpace.getColRow(self, pos)
			s = space.what(pos)
			if s:
				pick = (s[0], s[1:], pos)
		elif id == 1 or id == 2:
			layout = Layout(id)
			layout.setPieces((id == 1 and space.blacket1) or space.blacket2)
			s = layout.getPiece(pos)
			if s:
				pick = (str(id), s, None)
		return pick

	def pastePickImage(self, base, pick):
		bw = pick[0]
		pc = pick[1]
		pos = pick[2]
		x, y = self.getAbsPosition(pos)
		pieces = (bw == '1' and self.blacks) or self.whites
		img = pieces[pc]
		box = img.point(c_hilight)
		base.paste(box, (x + 3, y + 5), img)

	def getBlacketImg(self, space, bw=1, pick=None):
		img = None
		if bw == 1:
			layout = Layout(1)
			layout.setPieces(space.blacket1)
			img = PIL.Image.open(os.path.join(path, 'blacket1.jpg'), 'r')
			img = layout.getImage(img, c_blacks)
			if pick and not pick[2] and pick[0] == '1':
				layout.pickImage(img, c_blacks, pick[1])
		else:
			layout = Layout(2)
			layout.setPieces(space.blacket2)
			img = PIL.Image.open(os.path.join(path, 'blacket2.jpg'), 'r')
			img = layout.getImage(img, c_whites)
			if pick and not pick[2] and pick[0] == '2':
				layout.pickImage(img, c_whites, pick[1])
		return img

